import { Text, VStack, Widget, StyledText, HStack, Image, Spacer, DynamicShapeStyle, fetch, Link } from "scripting"

export const colorTitle: DynamicShapeStyle = {
    light: "#1a1a1a",
    dark: "#F5F5F5"
  }

export const colorDescription: DynamicShapeStyle = {
    light: "#242424",
    dark: "#CCCCCC"
  }

export const colorBackground: DynamicShapeStyle = {
    light: "#F0f0f0",
    dark: "#000000"
  }

const itemTitle: StyledText = {
    font: "callout",
    fontWeight: "semibold",
    content: "加载中...",
}

const itemDomain: StyledText = {
    fontWeight: "regular",
    font: "caption2",
    content: "...",
    foregroundColor: "#999999"
}

const itemDescription: StyledText = {
    fontWeight: "regular",
    font: "footnote",
    content: "...",
}

const itemTime: StyledText = {
    fontWeight: "regular",
    font: "caption2",
    content: "...",
    foregroundColor: "#999999"
}

export interface WidgetDataProps {
    title: string;
    domain: string;
    description: string;
    time: string;
}

let itemUrl: string = "";

export function WidgetView() {
  // 检查是否为 large 或 extra large 尺寸
  const useLargeLayout = Widget.family === "systemLarge" || Widget.family === "systemExtraLarge";

  const currentItemTitleStyle: StyledText = useLargeLayout 
    ? { ...itemTitle, font: "title2" } 
    : itemTitle;
  const currentItemDomainStyle: StyledText = useLargeLayout 
    ? { ...itemDomain, font: "footnote" } 
    : itemDomain;
  const currentItemDescriptionStyle: StyledText = useLargeLayout 
    ? { ...itemDescription, font: "callout" } 
    : itemDescription;
  const currentItemTimeStyle: StyledText = useLargeLayout 
    ? { ...itemTime, font: "footnote" } 
    : itemTime;

  // 根据尺寸调整行数限制 (可选)
  const titleLineLimit = useLargeLayout ? 3 : 2;
  const domainLineLimit = 1; // 域名通常保持单行
  const descriptionLineLimit = useLargeLayout ? 5 : 2;

  const imageFrame = useLargeLayout ? { width: 32, height: 32 } : { width: 24, height: 24 };

  return <Link url={itemUrl}>
      <VStack 
          frame={Widget.displaySize} 
          background={colorBackground}
      >
          <VStack 
              padding={useLargeLayout ? 24 : 16} 
              alignment="leading"
          >
              <HStack alignment="top">
                  <Image
                      imageUrl="https://df.fenx.work/assets/df-logo-color.png"
                      scaleToFill={true}
                      aspectRatio={{ value: 1.0, contentMode:"fill" }}
                      resizable={true}
                      frame={imageFrame}
                      placeholder={<Text>df_logo</Text>}
                      />
                  <Spacer />
                  <Text styledText={currentItemTimeStyle} />
              </HStack>

              {useLargeLayout ? <Spacer minLength={8} /> : <Spacer /> }

              <VStack spacing={6} alignment="leading">
                  <Text 
                      foregroundStyle={colorTitle} 
                      lineLimit={titleLineLimit} 
                      styledText={currentItemTitleStyle} 
                  />
                  <Text 
                      lineLimit={domainLineLimit} 
                      styledText={currentItemDomainStyle} 
                  />
                  <Text 
                      foregroundStyle={colorDescription} 
                      lineLimit={descriptionLineLimit} 
                      styledText={currentItemDescriptionStyle} 
                  />
              </VStack>
          </VStack>
      </VStack>
  </Link>
}

async function main() {

    const url = "https://xuomfgjgmsoozzxnwtht.supabase.co/functions/v1/random-content-query";

    try {
        const response = await fetch(url);
        if (!response.ok) {
             throw new Error(`HTTP error! status: ${response.status}`);
        }
        const responseData = await response.json();

        if (responseData && Array.isArray(responseData) && responseData.length > 0) {
            const data = responseData[0];

            itemTitle.content = data.item_title || "获取标题失败";
            itemDomain.content = data.item_domain || "获取域名失败";
            itemDescription.content = data.item_desc || "获取描述失败";
            itemUrl = data.item_url || "";

            try {
                const publishDate = new Date(data.publish_at);
                const today = new Date();
                publishDate.setHours(0, 0, 0, 0);
                today.setHours(0, 0, 0, 0);
                const diffTime = Math.abs(today.getTime() - publishDate.getTime());
                const diffDays = Math.ceil(diffTime / (1000 * 60 * 60 * 24));
                itemTime.content = `${diffDays}天前`;
            } catch (timeError) {
                console.error("Error calculating time difference:", timeError);
                itemTime.content = "时间错误";
            }
        } else if (responseData) {
            console.error("Unexpected response format:", responseData);
            itemTitle.content = "数据格式错误";
            itemDescription.content = "收到的数据不是预期的数组格式。";
            itemDomain.content = "";
            itemTime.content = "";
        } else {
            itemTitle.content = "未能获取数据";
            itemDomain.content = "";
            itemDescription.content = "服务器返回了空数据";
            itemTime.content = "";
        }

    } catch (error) {
        console.error("Failed to fetch or process data:", error);
        itemTitle.content = "请求或处理失败";
        itemDomain.content = "";
        itemDescription.content = String(error);
        itemTime.content = "";
    }

    // 暂定刷新逻辑
    const nextRefresh = new Date();
    nextRefresh.setHours(nextRefresh.getHours() + 1);
    // @ts-ignore
    Widget.refreshAfterDate = nextRefresh; 

    // 渲染小组件
    Widget.present(<WidgetView />);
}

main();